"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimulationBuilder = void 0;
var matterjs_physics_1 = require("../physics/matterjs_physics");
var svg_data_1 = require("../base/svg_data");
var command_mapper_1 = require("../base/command_mapper");
var svg_path_parser_1 = require("svg-path-parser");
var gravity_plugin_1 = require("../physics/plugins/gravity_plugin");
var view_collider_border_plugin_1 = require("../physics/plugins/view_collider_border_plugin");
var mouse_plugin_1 = require("../physics/plugins/mouse_plugin");
var renderer_plugin_1 = require("../physics/plugins/renderer_plugin");
var production_renderer_1 = require("../renderer/production_renderer");
var scroll_plugin_1 = require("../physics/plugins/scroll_plugin");
var limit_velocity_plugin_1 = require("../physics/plugins/limit_velocity_plugin");
var limit_angular_velocity_plugin_1 = require("../physics/plugins/limit_angular_velocity_plugin");
var SimulationBuilder = /** @class */ (function () {
    function SimulationBuilder(aggregator, svg) {
        this._aggregator = aggregator;
        this._svg = svg;
        this._plugins = [];
    }
    SimulationBuilder.prototype.withGravity = function (gravity) {
        this._plugins.push(new gravity_plugin_1.GravityPlugin(gravity));
        return this;
    };
    SimulationBuilder.prototype.withBorderCollider = function () {
        this._plugins.push(new view_collider_border_plugin_1.ViewColliderBorder());
        return this;
    };
    SimulationBuilder.prototype.withMouseInteraction = function (mouseForce) {
        this._plugins.push(new mouse_plugin_1.MousePlugin(mouseForce));
        return this;
    };
    SimulationBuilder.prototype.withScrollInteraction = function (scrollForce) {
        this._plugins.push(new scroll_plugin_1.ScrollPlugin(scrollForce));
        return this;
    };
    SimulationBuilder.prototype.withDebugRenderer = function () {
        this._plugins.push(new renderer_plugin_1.RendererPlugin());
        return this;
    };
    SimulationBuilder.prototype.withCustomPlugin = function (plugin) {
        this._plugins.push(plugin);
        return this;
    };
    SimulationBuilder.prototype.withLimitVelocityPlugin = function (maxSpeed) {
        this._plugins.push(new limit_velocity_plugin_1.LimitVelocityPlugin(maxSpeed));
        return this;
    };
    SimulationBuilder.prototype.withLimitAngularVelocityPlugin = function (maxSpeed) {
        this._plugins.push(new limit_angular_velocity_plugin_1.LimitAngularVelocityPlugin(maxSpeed));
        return this;
    };
    SimulationBuilder.prototype.startSimulation = function (config) {
        var svgData = this.createSvgData();
        var physics = new matterjs_physics_1.MatterJsPhysics(svgData, config, this._plugins);
        var renderer = new production_renderer_1.ProductionRenderer();
        renderer.renderLoop(svgData, physics);
    };
    SimulationBuilder.prototype.createSvgData = function () {
        var paths = Array.prototype.slice.call(this._svg.getElementsByTagName('path'));
        var pathCommandArray = paths
            .map(function (path) { return path.getAttribute("d"); })
            .map(function (pathString) { return (0, command_mapper_1.expandCommands)((0, svg_path_parser_1.makeAbsolute)((0, svg_path_parser_1.parseSVG)(pathString))); });
        var aggregatedCommands = this._aggregator.aggregate(pathCommandArray);
        return new svg_data_1.SvgData(this._svg, pathCommandArray, aggregatedCommands);
    };
    return SimulationBuilder;
}());
exports.SimulationBuilder = SimulationBuilder;
